/*
 * ContorollerDefinition: コントローラ定義 
 */

package org.dbc_works.app.sbfiles;

import java.awt.Component;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.util.Properties;

import org.dbc_works.lib.ui.ActionDefinition;
import org.dbc_works.lib.ui.ActionManager;
import org.dbc_works.lib.ui.MenuItemDefinition;

import org.dbc_works.app.sbfiles.view.LabelManager;

final public class ControllerDefinition
{
	//
	// 定数
	//

	// 「編集」メニューのメニューバー上のインデックス
	public static final int MENU_EDIT = 1;

	//
	// コンストラクタ
	//
	ControllerDefinition() {
	}

	//
	// 操作
	//

	// コントローラを作成する
	Component[] buildMainController(
		final MainFrame mainFrame,		// アクションを受けとるメインフレーム
		final Frame frame, 			// メニューを作成するフレーム
		final ActionManager manager)	// アクションマネージャー
	{
		final Properties menuLabelProperties = LabelManager.getMenuLabelProperties();

		final Toolkit tk = Toolkit.getDefaultToolkit();

		// ツールバー
		final ActionDefinition[] actionDef = new ActionDefinition[] {
			// ソート
			new ActionDefinition(
				mainFrame.sortAction,
				menuLabelProperties.getProperty(LabelManager.MENU_FILE_SORT),
				tk.getImage(getClass().getResource("/cmd-sort.png")),
				ActionDefinition.Style.NORMAL),
			// 連番設定
			new ActionDefinition(
				mainFrame.numberingAction,
				menuLabelProperties.getProperty(LabelManager.MENU_FILE_NUMBERING),
				tk.getImage(getClass().getResource("/cmd-numbering.png")),
				ActionDefinition.Style.NORMAL),
			// GZIP圧縮
			new ActionDefinition(
				mainFrame.gzipAction,
				menuLabelProperties.getProperty(LabelManager.MENU_FILE_GZIP),
				tk.getImage(getClass().getResource("/cmd-gzip.png")),
				ActionDefinition.Style.NORMAL),
			// GZIP伸長
			new ActionDefinition(
				mainFrame.gunzipAction,
				menuLabelProperties.getProperty(LabelManager.MENU_FILE_GUNZIP),
				tk.getImage(getClass().getResource("/cmd-gunzip.png")),
				ActionDefinition.Style.NORMAL),
		};

		// 「ファイル」メニュー
		final MenuItemDefinition[] fileMenuDef =
			new MenuItemDefinition[] {
				new MenuItemDefinition(actionDef[0], KeyEvent.VK_S),
				new MenuItemDefinition(actionDef[1], KeyEvent.VK_N),
				MenuItemDefinition.SEPARATOR,
				new MenuItemDefinition(actionDef[2], KeyEvent.VK_G),
				new MenuItemDefinition(actionDef[3], KeyEvent.VK_U),
				MenuItemDefinition.SEPARATOR,
				new MenuItemDefinition(
					new ActionDefinition(
						mainFrame.closeAction,
						menuLabelProperties.getProperty(
							LabelManager.MENU_FILE_EXIT),
						null,
						ActionDefinition.Style.NORMAL),
					KeyEvent.VK_X),
			};

		// 「編集」メニュー
		final MenuItemDefinition[] editMenuDef =
			new MenuItemDefinition[] {
				new MenuItemDefinition(
					new ActionDefinition(
						mainFrame.undoAction,
						menuLabelProperties.getProperty(
							LabelManager.MENU_EDIT_UNDO),
						null,
						ActionDefinition.Style.NORMAL),
					KeyEvent.VK_Z,
					KeyEvent.VK_Z),
				new MenuItemDefinition(
					new ActionDefinition(
						mainFrame.redoAction,
						menuLabelProperties.getProperty(
							LabelManager.MENU_EDIT_REDO),
						null,
						ActionDefinition.Style.NORMAL),
					KeyEvent.VK_Y,
					KeyEvent.VK_Y),
				};

		manager.buildMenu(
			menuLabelProperties.getProperty(LabelManager.MENU_FILE),
			KeyEvent.VK_F,
			fileMenuDef,
			frame);
		manager.buildMenu(
			menuLabelProperties.getProperty(LabelManager.MENU_EDIT),
			KeyEvent.VK_E,
			editMenuDef,
			frame);

		return manager.createToolBarComponents(actionDef);
	}
}
