/*
 * CountReporter: 
 * 注記: ObserverパターンのObservableクラス
 */
package org.dbc_works.app.sbfiles;

import java.util.Observable;
import java.util.Observer;

public final class CountReporter extends Observable
	implements MainFrame.UpdateArgument
{
	//
	// 属性
	//
	
	private final int	totalCount;
	private int	currentCount;

	//
	// コンストラクタ
	//
	public CountReporter(
		Observer observer,	// レポート受信クラス 
		int nTotalCount)	// 最大数
	{
		if (nTotalCount <= 0) {
			throw new IllegalArgumentException();
		}
		if (observer == null) {
			throw new NullPointerException();
		}
		
		totalCount = nTotalCount;
		addObserver(observer);
		resetCount();
	}

	//
	// 操作
	// 

	// カウントをリセットする
	public void resetCount()
	{
		currentCount = 0;
		setChanged();
		notifyObservers(this);
	}
	
	// カウントアップを報告する
	public void reportCountUp()
	{
		if (totalCount <= currentCount) {
			throw new IllegalStateException();
		}

		++currentCount;
		setChanged();
		notifyObservers(this);
	}

	//
	// 再定義メソッド(MainFrame.UpdateArgument)
	// 

	// 総数を返す
	public int getTotalCount()
	{
		return totalCount;
	}

	// 処理数を返す
	public int getProcessCount()
	{
		return currentCount;
	}

}
