/*
 * ProcessEachEntryCommand: 
 */

package org.dbc_works.app.sbfiles.cmd;

import java.io.File;
import java.io.IOException;
import java.io.Writer;

import org.dbc_works.lib.model.Command;
import org.dbc_works.lib.yaml.YamlSerializer;

import org.dbc_works.app.sbfiles.CountReporter;
import org.dbc_works.app.sbfiles.MainFrame;

abstract class ProcessEachEntryCommand extends SfCommand
{
	//
	// コンストラクタ
	//
	protected ProcessEachEntryCommand(
		String strCommandName,	// 
		File fileTarget,		// 対象ディレクトリ
		MainFrame frame)		//
	{
		super(strCommandName, fileTarget, frame);
	}

	//
	// 再定義メソッド(実装)
	//

	// 実行する	
	final protected Command.Result doExecute()
		throws Exception
	{
		final CountReporter reporter = new CountReporter(getObserver(), getProcessEntryCount());
		final Writer logWriter = getLogWriter();
		YamlSerializer logSerializer = null;
		try {
			logSerializer = getLogSerializer(getOperationName(), logWriter);
			logSerializer.startWriteMapping(LOG_KEY_RESULT);
			logSerializer.startChildNode();

			process(logSerializer, reporter);
		}
		catch (IOException e) {
			logSerializer.endChildNode();
			logSerializer.startWriteMapping("Cancel");
			logSerializer.startChildNode();
			cancelProcess(logSerializer, null);
			throw e;
		}
		finally {
			if (logSerializer != null) {
				logSerializer.endChildNode();
			}
			if (logWriter != null) {
				try {
					logWriter.close();
				}
				catch (IOException e) {
					// Do nothing.
				}
			}
		}

		return Command.Result.SUCCESS;
	}

	// 実行を取り消す
	final protected Command.Result doCancel()
		throws Exception
	{
		final CountReporter reporter = new CountReporter(getObserver(), getProcessEntryCount());
		final Writer logWriter = getLogWriter();
		YamlSerializer logSerializer = null;
		try {
			logSerializer = getLogSerializer(getOperationName(), logWriter);
			logSerializer.startWriteMapping(LOG_KEY_RESULT);
			logSerializer.startChildNode();

			cancelProcess(logSerializer, reporter);
		}
		finally {
			if (logSerializer != null) {
				logSerializer.endChildNode();
			}
			if (logWriter != null) {
				try {
					logWriter.close();
				}
				catch (IOException e) {
					// Do nothing.
				}
			}
		}

		return Command.Result.SUCCESS;
	}
	
	//
	// 再定義必須メソッド(実装)
	//
	
	abstract protected int getProcessEntryCount();
	abstract protected String getOperationName();
	abstract protected void process(YamlSerializer logSerializer, CountReporter reporter) throws IOException;
	abstract protected void cancelProcess(YamlSerializer logSerializer, CountReporter reporter) throws IOException;
}
