/*
 * UpdateEntryOrderCommand: エントリソートコマンド 
 */

package org.dbc_works.app.sbfiles.cmd;
import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import org.dbc_works.app.sbfiles.FatEntryOrderChanger;
import org.dbc_works.app.sbfiles.MainFrame;
import org.dbc_works.lib.model.Command;
import org.dbc_works.lib.yaml.YamlSerializer;

public final class UpdateEntryOrderCommand extends SfCommand
{
	//
	// 定数
	//
	
	private static final String LOG_KEY_AFTER = "after";
	private static final String LOG_KEY_BEFORE = "before";
	private static final String LOG_VALUE_OPERATION = "update entry order";

	//
	// クラスメソッド(実装)
	//
	
	// 
	static private Command.Result getCommandResult(
		FatEntryOrderChanger.Result result)	//
	{
		if (result == null) {
			throw new NullPointerException();
		}

		Command.Result commandResult = Result.SUCCESS;
		
		final Map resultMap = new HashMap();
		resultMap.put(FatEntryOrderChanger.Result.FAIL_CREATE_TEMP_DIR,
						Result.FAIL_CREATE_TEMP_DIR);
		resultMap.put(FatEntryOrderChanger.Result.FAIL_MOVE_ENTRY,
						Result.FAIL_MOVE_ENTRY);
		resultMap.put(FatEntryOrderChanger.Result.FAIL_DELETE_TEMP_DIR,
						Result.FAIL_DELETE_TEMP_DIR);

		if (resultMap.containsKey(result)) {
			commandResult = (Command.Result)resultMap.get(result);
		}
		else if (result != FatEntryOrderChanger.Result.SUCCESS) {
			commandResult = Result.FAIL_UNKOWN;
		}
	/*				
		if (result == FatEntryOrderChanger.Result.FAIL_CREATE_TEMP_DIR) {
			commandResult = Result.FAIL_CREATE_TEMP_DIR;
		}
		else if (result == FatEntryOrderChanger.Result.FAIL_MOVE_ENTRY) {
			commandResult = Result.FAIL_MOVE_ENTRY;
		}
		else if (result == FatEntryOrderChanger.Result.FAIL_DELETE_TEMP_DIR) {
			commandResult = Result.FAIL_DELETE_TEMP_DIR;
		}
		else if (result != FatEntryOrderChanger.Result.SUCCESS) {
			commandResult = Result.FAIL_UNKOWN;
		}
	 */
		return commandResult;
	}
	
	private static void recordLog(
		String strKey,					//
		File[] afileRecord,				//
		YamlSerializer logSerializer)	//
		throws IOException
	{
		if (strKey == null) {
			throw new NullPointerException();
		}
		else if (strKey.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (afileRecord == null) {
			throw new NullPointerException();
		}
		if (logSerializer == null) {
			throw new NullPointerException();
		}

		logSerializer.startWriteMapping(strKey);
		logSerializer.startChildNode();
		for (int nIndex = 0; nIndex < afileRecord.length; nIndex++) {
			logSerializer.writeSequence(afileRecord[nIndex].getName());
		}
		logSerializer.endChildNode();
	}

	//
	// 属性
	//

	// オリジナルの並び順
	final private File[]	originalEntries;

	// 更新後の並び順
	final private File[]	updateEntries;

	//
	// コンストラクタ
	//
	public UpdateEntryOrderCommand(
		String strCommandName,		// コマンド名
		File fileTarget,			// 対象ディレクトリ
		File[] fileUpdateEntries,	// 更新エントリ 
		MainFrame frame)			//
	{
		super(strCommandName, fileTarget, frame);

		updateEntries = fileUpdateEntries;
		originalEntries = fileTarget.listFiles();
	}
	
	//
	// 再定義メソッド(状態)
	//

	// 実行可能かを返す
	public boolean canExecute()
	{
		// 常に実行可能
		return true;
	}

	//
	// 再定義メソッド(実装)
	//

	// 実行する	
	protected Command.Result doExecute()
		throws Exception
	{
		Command.Result result;
		
		final Writer logWriter = getLogWriter();
		YamlSerializer logSerializer = null;
		try {
			logSerializer = getLogSerializer(LOG_VALUE_OPERATION, logWriter);
			recordLog(LOG_KEY_BEFORE, originalEntries, logSerializer);

			final FatEntryOrderChanger changer = new FatEntryOrderChanger(targetDirectory,
																			updateEntries,
																			getObserver());
			result = getCommandResult(changer.execute());

			recordLog(LOG_KEY_AFTER, updateEntries, logSerializer);
		}
		finally {
			if (logWriter != null) {
				try {
					logWriter.close();
				}
				catch (IOException e) {
					// Do nothing.
				}
			}
		}

		return result;
	}

	// 実行を取り消す
	protected Command.Result doCancel()
		throws Exception
	{
		Command.Result result;
		
		final Writer logWriter = getLogWriter();
		YamlSerializer logSerializer = null;
		try {
			logSerializer = getLogSerializer(LOG_VALUE_OPERATION, logWriter);
			recordLog(LOG_KEY_BEFORE, updateEntries, logSerializer);

			final FatEntryOrderChanger changer = new FatEntryOrderChanger(targetDirectory,
																			originalEntries,
																			getObserver());
			result = getCommandResult(changer.execute());

			recordLog(LOG_KEY_AFTER, originalEntries, logSerializer);
		}
		finally {
			if (logWriter != null) {
				try {
					logWriter.close();
				}
				catch (IOException e) {
					// Do nothing.
				}
			}
		}
		
		return result;
	}
}
