/*
 * HeaderField:
 * 注記: 実装は仕様を完全に満たすものではない 
 */
package org.dbc_works.lib.mime;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public final class HeaderField
{
	//
	// 定数
	//

	public static final String HEADER_DATE	= "Date";

	public static final String HEADER_CONTENTTYPE	= "ContentType";
	public static final String CONTENTTYPE_MULTIPART_MIXED	= "multipart/mixed";

	private static final char DELIMIT_NAME	= ':';
	private static final char DELIMIT_PARAMETER	= ';';
	private static final char DELIMIT_PARAMETER_NAME	= '=';

	//
	// クラスメソッド(公開)
	//

	// 解析可能かを返す
	public static boolean canParse(
		String strSourceFirstLine)	// 先頭行
	{
		if (strSourceFirstLine == null) {
			throw new NullPointerException();
		}

		boolean bResult = false;
		if (0 < strSourceFirstLine.length()) {
			bResult = (strSourceFirstLine.charAt(0) != ' '
					&& 0 < strSourceFirstLine.indexOf(DELIMIT_NAME));  
		}
		return bResult;
	}
	
	// 解析する
	public static HeaderField parse(
		String strSourceFirstLine)	// 解析対象行
	{
		if (canParse(strSourceFirstLine) == false) {
			throw new IllegalArgumentException();
		}

		int nNamePos = strSourceFirstLine.indexOf(DELIMIT_NAME);
		int nParamPos = strSourceFirstLine.indexOf(DELIMIT_PARAMETER);
		String strName = strSourceFirstLine.substring(0, nNamePos);
		String strValue = (nParamPos < nNamePos
						? strSourceFirstLine.substring(nNamePos + 1).trim()
						: strSourceFirstLine.substring(nNamePos + 1, nParamPos).trim());

		HeaderField header = new HeaderField(strName, strValue);
		if (nNamePos < nParamPos) {
			String strParameters = strSourceFirstLine.substring(nParamPos + 1);
			nParamPos = indexOfParameterDelimit(strParameters);
			while (0 <= nParamPos) {
				parseParameter(strParameters.substring(0, nParamPos), header.parameters);
				strParameters = strParameters.substring(nParamPos + 1);
				nParamPos = strParameters.indexOf(DELIMIT_PARAMETER);
			}
			strParameters = strParameters.trim();
			if (0 < strParameters.length()) {
				parseParameter(strParameters, header.parameters);
			}
		}

		return header;
	}

	//
	// クラスメソッド(実装)
	//

	// パラメータ文字列の区切り文字の位置を返す
	private static int indexOfParameterDelimit(
		String strParameters)	// 解析対象パラメータ文字列
	{
		int nDelimitIndex = -1;
		boolean bInQuote = false; 
		for(int nIndex = 0; nIndex < strParameters.length(); nIndex++) {
			if (strParameters.charAt(nIndex) == DELIMIT_PARAMETER) {
				if (bInQuote == false) {
					break;
				}
			}
			else if (strParameters.charAt(nIndex) == '"') {
				bInQuote = !bInQuote;
			}
		}
		return nDelimitIndex;
	}

	// パラメータを解析する
	private static void parseParameter(
		String strParameter,	// 解析対象パラメータ文字列
		Map parameters)			// パラメータの集合
	{
		if (strParameter == null || parameters == null) {
			throw new NullPointerException();
		}
		if (strParameter.length() == 0) {
			throw new IllegalArgumentException();
		}
		
		int nSplitPos = strParameter.indexOf(DELIMIT_PARAMETER_NAME);
		if (nSplitPos <= 0) {
			throw new IllegalArgumentException();
		}
		String strName = strParameter.substring(0, nSplitPos).trim();
		if (strName.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (parameters.containsKey(strName)) {
			throw new IllegalArgumentException();
		}

		StringBuffer sbValue = new StringBuffer(strParameter.substring(nSplitPos + 1).trim());
		if (sbValue.charAt(0) == '"') {
			int nLastIndex = sbValue.length() - 1;
			if (sbValue.charAt(nLastIndex) != '"') {
				throw new IllegalArgumentException();
			}
			sbValue.deleteCharAt(nLastIndex);
			sbValue.deleteCharAt(0);
		}
		parameters.put(strName, sbValue.toString());
	}

	//
	// 属性
	//

	public final String name;
	public final String value;

	private final Map parameters	= new HashMap();

	//
	// コンストラクタ
	//
	public HeaderField(
		String strName,		// ヘッダー名
		String strValue)	// 値
	{
		if (strName == null) {
			throw new NullPointerException();
		}
		else if (strName.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (strValue == null) {
			throw new NullPointerException();
		}

		name = strName;
		value = strValue;
	}

	//
	// 再定義メソッド(状態)
	//

	// 文字列表現を返す
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append(name);
		sb.append(DELIMIT_NAME);
		sb.append(' ');
		sb.append(value);

		Iterator it = parameters.keySet().iterator();
		while (it.hasNext()) {
			sb.append(DELIMIT_PARAMETER);
			
			String strName = (String)it.next();
			sb.append(strName);
			sb.append(DELIMIT_PARAMETER_NAME);
			sb.append('"');
			sb.append(parameters.get(strName).toString());
			sb.append('"');
		}

		return sb.toString();
	}

	//
	// 操作
	//

	// パラメータを設定する
	public void setParameter(
		String strName,			// 名前
		Object parameterValue)	// 値
	{
		if (strName == null || parameterValue == null) {
			throw new NullPointerException();
		}
		if (strName.length() == 0) {
			throw new IllegalArgumentException();
		}

		parameters.put(strName, parameterValue);
	}
}
