/*
 * ActionManager: アクションマネージャ 
 */
package org.dbc_works.lib.ui;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ItemListener;
import java.util.EventListener;
import java.util.HashMap;
import java.util.Map;

public abstract class ActionManager
{
	//
	// クラス属性
	//

	// デバッグフラグ
	protected static final boolean DEBUG	= false;

	//
	// クラスメソッド(公開)
	//

	// インスタンスを作成する
	// 注記: J2MEに存在しないクラスのロードを避けるためにリフレクションで作成する
	// Factory Methodパターンの退化形
	public static ActionManager createInstance(
		boolean bForAwt)	// AWT用作成フラグ
		throws ClassNotFoundException, IllegalAccessException, InstantiationException
	{
		ActionManager manager;
		
		if (bForAwt) {
			manager = new org.dbc_works.lib.ui.awt.ActionManager();
		//	manager = (ActionManager)Class.forName("org.dbc_works.lib.ui.awt.ActionManager").newInstance();
		}
		else {
		//	controller = new ui.swing.ActionController();
			manager = (ActionManager)Class.forName("org.dbc_works.lib.ui.swing.ActionManager").newInstance();
		}

		return manager;
	}

	// メニューの表記を返す
	// 注記: Swing用の表記のアクセラレータ用文字列を削除するための処理
	protected static String getStrictLabel(
		String strOriginalLabel)	// 元のラベル
	{
		String strLabel = strOriginalLabel;
		final int nClose = strLabel.lastIndexOf(')');
		if (0 < nClose) {
			final int nOpen = strLabel.lastIndexOf('(');
			if (0 < nOpen && (nClose - nOpen) == 2) {
				strLabel = strOriginalLabel.substring(0, nOpen)
							+ strOriginalLabel.substring(nClose + 1);
			}
		}
		return strLabel;
	}

	//
	// 属性
	//
	private final Map menuItemMap	= new HashMap();
	private final Map buttonMap	= new HashMap();

	//
	// コンストラクタ
	//
	protected ActionManager()
	{
	}

	//
	// 操作
	//

	// ツールバー用コンポーネントを作成する
	public final Component[] createToolBarComponents(
		ActionDefinition[] actionDefinitions)
	{
		final Component[] components = new Component[actionDefinitions.length];
	
		for (int nIndex = 0; nIndex < actionDefinitions.length; nIndex++) {
			final Component component = createToolBarComponent(actionDefinitions[nIndex]);
			components[nIndex] = component;
			getButtonMap().put(actionDefinitions[nIndex].listener, component);
		}
		return components;
	}

	//
	// 実装
	//

	// アクションとメニュー項目のマップを返す
	protected Map getMenuItemMap()
	{
		return menuItemMap; 
	}

	// アクションとボタンのマップを返す
	protected Map getButtonMap()
	{
		return buttonMap; 
	}

	//
	// 再定義必須メソッド(操作)
	//

	// メニューを作成する
	abstract public void buildMenu(String strMenu,
										int nMenuMnemonic,
										MenuItemDefinition[] menuItemDefinitions,
										Frame frameTarget);

	// 有効 / 無効を設定する
	abstract public void setEnable(EventListener listener, boolean bEnable);

	// チェック状態を設定する
	abstract public void setCheckStatus(ItemListener listener, boolean bCheck);

	//
	// 再定義必須メソッド(実装)
	//

	// ツールバー用コンポーネントを作成する
	abstract protected Component createToolBarComponent(ActionDefinition actionDefinition);
}
