/*
 * DirectoryChooser: AWT版ディレクトリ選択ダイアログ 
 */
package org.dbc_works.lib.ui.awt;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;

// 注記: 現在リストボックスに設定するパスは絶対パスにしているが、単独の
// ディレクトリ名表示にしたほうがいいかもしれない。そのときは全体の構成を
// 見直したほうがいいかもしれない。
public final class DirectoryChooser extends Dialog
	implements ActionListener
{
	//
	// 属性
	//

	private boolean result;
	private String path;

	private TextField pathText;
	private java.awt.List subDirList;
	private Button okButton;
	private Button cancelButton;

	//
	// コンストラクタ
	//
	public DirectoryChooser(
		Frame owner,		// 親フレーム 
		String title,		// ウィンドウタイトル
		boolean modal,		// モーダル
		String initialPath)	// 初期化パス
	{
		super(owner, title, modal);

		if (initialPath != null && 0 < initialPath.length()) {
			path = initialPath; 
		}
		else {
			path = "";
		}

		init();
	}

	//
	// 状態
	//

	// 問いあわせた結果を返す
	public boolean getResult()
	{
		return result;
	}
	
	// パスを返す
	public String getPath()
	{
		return path;
	}

	//
	// 実装
	//

	// 初期化する
	private void init()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				close();
			}
		});

		// コントロールを作成する

		// 上部
		Container northPanel = new Container();
		northPanel.setLayout(new BorderLayout());

		northPanel.add(new Label("Path:"), BorderLayout.WEST);

		pathText = new TextField(30);
		pathText.setText(path);
		northPanel.add(pathText, BorderLayout.CENTER);

		Container northButtonPanel = new Container();
		northButtonPanel.setLayout(new GridLayout(1, 2));

		Button refButton = new Button("List");
		refButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				setSubDirectory();
			}
		});
		
		northButtonPanel.add(refButton);

		Button upDirButton = new Button("Up");
		upDirButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				moveToParentDirectory();
			}
		});
		northButtonPanel.add(upDirButton);
		
		northPanel.add(northButtonPanel, BorderLayout.EAST);

		add(northPanel, BorderLayout.NORTH);
		
		// 中央部
		subDirList = new java.awt.List(10);
		subDirList.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				setSelectedPathToTextField();
				setSubDirectory();
			}
		});
		subDirList.addItemListener(new ItemListener() {
			public void itemStateChanged(
				ItemEvent e) {
				setSelectedPathToTextField();
			}
		});
		add(subDirList, BorderLayout.CENTER);

		// 下部		
		okButton = DialogUtilities.createOkButton(this);
		cancelButton = DialogUtilities.createCancelButton(this);

		Container southPanel = new Container();
		southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		Button[] buttons = new Button[] { okButton, cancelButton, };
		southPanel.add(DialogUtilities.createResponseButtonContainer(buttons));
		add(southPanel, BorderLayout.SOUTH);

		pack();

		setSubDirectory();
	}

	// サブディレクトリ一覧リストにサブディレクトリを設定する
	private void setSubDirectory()
	{
		String strPath = pathText.getText();
		if (strPath.length() == 0) {
			return;
		}

		File inputPathFile = new File(strPath);
		if (inputPathFile.exists() && inputPathFile.isDirectory()) {
			subDirList.removeAll();
			File[] files = inputPathFile.listFiles();
			
			ArrayList dirNamesList = new ArrayList();
			
			for (int nIndex = 0; nIndex < files.length; nIndex++) {
				if (files[nIndex].isDirectory()) {
				//	dirNamesList.add(files[nIndex].getName());
					dirNamesList.add(files[nIndex].getAbsolutePath());
				}
			}

			Object[] directoryNames = dirNamesList.toArray(); 
			Arrays.sort(directoryNames);
			for (int nIndex = 0; nIndex < directoryNames.length; nIndex++) {
				subDirList.add((String)directoryNames[nIndex]);
			}
		}
		else {
			getToolkit().beep();
		}
	}

	// パス入力テキストフィールドにリストの選択を設定する
	private void setSelectedPathToTextField()
	{
		pathText.setText(subDirList.getSelectedItem());
	}
	
	// 現在の選択ディレクトリの親ディレクトリに移動する
	private void moveToParentDirectory()
	{
		String strPath = pathText.getText();
		if (strPath.length() == 0) {
			return;
		}

		int nIndex = strPath.lastIndexOf(File.separatorChar);
		if (0 <= nIndex) {
			if (strPath.lastIndexOf(File.separatorChar, nIndex - 1) < 0) {
				nIndex++;
			}

			strPath = strPath.substring(0, nIndex);
			if (strPath.compareToIgnoreCase(pathText.getText()) != 0) {
				pathText.setText(strPath);
				setSubDirectory();
			}
		}
		else {
			getToolkit().beep();
		}
	}
	
	// ダイアログを閉じる
	private void close()
	{
		hide();
		dispose();
	}

	//
	// 再定義メソッド(ActionListener)
	//

	// ボタン押し下げ処理を行う
	public void actionPerformed(
		ActionEvent e)	// イベント情報(入力)
	{
		result = (e.getSource() == okButton);
		if (result) {
			path = pathText.getText();
		}

		close();
	}
}
