/*
 * ZipUtilities: 圧縮ファイルユーティリティ
 */
package org.dbc_works.lib.util.zip;

import java.io.File;

import org.dbc_works.lib.io.FileUtilities;

public final class ZipUtilities
{
	//
	// 内部型定義
	//

	// FileType: 圧縮ファイル種別
	// 『Effective Java』「項目21 enum構文をクラスで置き換える」に従って実装
	static public final class FileType
	{
		//
		// クラス属性
		//

		public static final FileType NOTZIP	= new FileType("NotZip", "");
		public static final FileType GZIP	= new FileType("GZip", "gz");
		public static final FileType ZIP	= new FileType("Zip", "zip");

		//
		// 属性
		//
		public final String extension;

		private final String name;

		//
		// コンストラクタ
		//
		private FileType(
			String strName,	// 名前
			String strExt)	// 拡張子
		{
			 name = strName;
			 extension = strExt;
		}

		//
		// 再定義メソッド(Object)
		//

		// 文字列化した結果を返す
		public String toString()
		{
			return name;
		}
	}

	//
	// クラスメソッド(公開)
	//

	// 圧縮ファイル種別を返す
	public static FileType getFileType(
		String entryName)	// 判定ファイル名
	{
		if (entryName == null) {
			throw new NullPointerException();
		}
		else if (entryName.length() == 0) {
			throw new IllegalArgumentException();
		}
		
		FileType type = FileType.NOTZIP;
		String strCompareExt = FileUtilities.getExtension(entryName);
		FileType[] types = new FileType[] {
			FileType.GZIP,			
			FileType.ZIP,			
		};
		for (int index = 0; index < types.length; index++) {
			if (strCompareExt.compareToIgnoreCase(types[index].extension) == 0) {
				type = types[index];
				break;
			}
		}

		return type;		
	}
	public static FileType getFileType(
		File file)	// 判定ファイルのパス
	{
		if (file == null) {
			throw new NullPointerException();
		}
		else if (file.exists() == false || file.isFile() == false) {
			throw new IllegalArgumentException();
		}
		
		return getFileType(file.getName());
	}
	
	//
	// コンストラクタ
	//
	private ZipUtilities()
	{
	}
}
