/*
 * YamlSerialiser:
 * 注記: 実装は必要最小限。ブロック形式のみサポート。 
 */
package org.dbc_works.lib.yaml;

import java.io.IOException;
import java.io.Writer;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Set;

public final class YamlSerializer
{
	//
	// クラス属性
	//

	// 改行コード
	private static final String NEWLINE = System.getProperty("line.separator");

	//
	// 属性
	//

	// 書き込み対象ライター
	private final Writer writer;

	// 子ノードレベル
	private int depth;

	// インデント文字
	private char[] indent;

	//
	// コンストラクタ
	//
	public YamlSerializer(
		Writer yamlWriter)	// 書きこみ対象ライター
		throws IOException
	{
		this(yamlWriter, false);
	}
	public YamlSerializer(
		Writer yamlWriter,			// 書きこみ対象ライター
		boolean bIgnoreStartDoc)	// 開始無視
		throws IOException
	{
		if (yamlWriter == null) {
			throw new NullPointerException();
		}
		writer = yamlWriter;
		depth = 0;
		
		if (bIgnoreStartDoc == false) {
			writeLine(Constants.DOCUMENT_SEPARATOR);
		}
	}

	//
	// 状態
	//

	// 子ノード書きこみ階層を返す
	public int getDepth()
	{
		return depth;
	}

	//
	// 操作
	//

	// 子ノード書きこみを開始する
	public void startChildNode()
		throws IOException
	{
		newLine();
		depth++;
		indent = new char[depth * 2];
		Arrays.fill(indent, ' ');
	}

	// 子ノード書きこみを終了する
	public void endChildNode()
	{
		if (depth == 0) {
			throw new IllegalStateException();
		}

		depth--;
		if (depth == 0) {
			indent = null;
		}
		else {
			indent = new char[depth * 2];
			Arrays.fill(indent, ' ');
		}
	}

	// Sequenceを書きこむ
	public void writeSequence(
		Object value)	// 値
		throws IOException
	{
		if (value == null) {
			throw new NullPointerException();
		}

		startWriteSequence();
		writer.write(value.toString());
		newLine();
	}
	public void writeSequences(
		Set set)	// 書きこみ対象集合
		throws IOException
	{
		if (set == null) {
			throw new NullPointerException();
		}

		Iterator it = set.iterator();
		while (it.hasNext()) {
			writeSequence(it.next());
		}
	}
	public void writeSequences(
		Object[] sequences)	// 書きこみ対象集合
		throws IOException
	{
		if (sequences == null) {
			throw new NullPointerException();
		}

		for (int nIndex = 0; nIndex < sequences.length; nIndex++) {
			writeSequence(sequences[nIndex].toString());
		}
	}

	// Sequence書きこみを開始する
	public void startWriteSequence()
		throws IOException
	{
		writeIndent();
		writer.write(Constants.MARKUP_SEQUENCE);
	}
	
	// Mappingを書きこむ
	public void writeMapping(
		String strKey,	// キー
		Object value)	// 値
		throws IOException
	{
		if (value == null) {
			throw new NullPointerException();
		}

		startWriteMapping(strKey);
		writer.write(' ');
		writer.write(value.toString());
		newLine();
	}

	// Mapping書きこみを開始する
	public void startWriteMapping(
		String strKey)	// キー
		throws IOException
	{
		if (strKey == null) {
			throw new NullPointerException();
		}
		else if (strKey.length() == 0) {
			throw new IllegalArgumentException();
		}
		
		writeIndent();
		writer.write(strKey);
		writer.write(Constants.DELIMIT_MAPPING);
	}

	// 改行する
	public void newLine()
		throws IOException
	{
		writer.write(NEWLINE);
	}

	// コメントを書きこむ
	public void writeComment(
		String strComment)	// コメント
		throws IOException
	{
		if (strComment == null) {
			throw new NullPointerException();
		}

		writer.write(Constants.DELIMIT_COMMENT);
		writer.write(' ');
		newLine();
	}
	
	//
	// 実装
	//

	// 行を書きこむ
	private void writeLine(
		String strWrite)	// 書きこみ対象文字列
		throws IOException
	{
		if (strWrite == null) {
			throw new NullPointerException();
		}

		writeIndent();
		writer.write(strWrite);
		newLine();
	}
	
	// インデントを書きこむ
	private void writeIndent()
		throws IOException
	{
		if (0 < depth) {
			writer.write(indent);
		}
	}
}
